from gpaw.dipole_correction import DipoleCorrection
from gpaw.mixer import Mixer, MixerSum, MixerDif
import numpy as np
from math import pi
from ase.constraints import FixAtoms
from ase.io import write, read
from gpaw.poisson import PoissonSolver
from gpaw import GPAW, FermiDirac, Mixer, MixerSum
from ase.optimize import BFGS
from ase.lattice.surface import *
from ase.structure import molecule
from ase.lattice.surface import add_adsorbate, fcc111
import sys
from sys import argv
from gpaw.utilities import h2gpts


name = sys.argv[1]
slab = read(name)
label = name[:-5] + '-re'

slab.pbc=(True,True,False)

for i in slab:                                                                                                                             
    if i.symbol == 'Ni':                                                                                                                   
        i.magmom=1.1                                                                                                                                                                                                          

poissonsolver = PoissonSolver()
correction = DipoleCorrection(poissonsolver,2)

calc = GPAW(h=0.20,
            occupations=FermiDirac(0.05),
            kpts = (3,2,1),
            spinpol=True,                                
            mixer=MixerDif(beta=0.02,nmaxold=3,weight=100.0),
            convergence={'eigenstates': 1e-6},
            #basis='dzp',                                  
            maxiter = 800,
            poissonsolver =correction,
            xc = 'RPBE',
            txt=label + '.txt')

slab.set_calculator(calc)
dyn=BFGS(slab,
         trajectory=label + '.traj',
         logfile=label + '.log')                                                                                                          
dyn.run(fmax=0.05)
calc.write(label + ".gpw")
